@@ -1,15 +1,12 @@
-#!/usr/bin/env python
-
 """
 This package implements various Nwchem Jobs and Error Handlers.
 """
 
-from __future__ import division
-
 __author__ = "Shyue Ping Ong"
 __copyright__ = "Copyright 2012, The Materials Project"
 __version__ = "0.1"
 __maintainer__ = "Shyue Ping Ong"
-__email__ = "shyuep@gmail.com"
+__email__ = "ongsp@ucsd.edu"
+__status__ = "Beta"
 __date__ = "6/17/13"
 
