@@ -20,19 +20,24 @@ import os
 import time
 import datetime
 import operator
+import shutil
+import glob
+
+import numpy as np
 
 from monty.dev import deprecated
+from monty.serialization import loadfn
 
 from math import ceil
 
 from custodian.custodian import ErrorHandler
 from custodian.utils import backup
-from pymatgen.io.vaspio.vasp_input import Poscar, VaspInput
+from pymatgen.io.vaspio.vasp_input import Poscar, VaspInput, Incar
 from pymatgen.transformations.standard_transformations import \
     PerturbStructureTransformation, SupercellTransformation
 
 from pymatgen.io.vaspio.vasp_output import Vasprun, Oszicar
-from custodian.ansible.intepreter import Modder
+from custodian.ansible.interpreter import Modder
 from custodian.ansible.actions import FileActions, DictActions
 
 
@@ -84,7 +89,7 @@ class VaspErrorHandler(ErrorHandler):
         self.errors = set()
 
     def check(self):
-	vi = VaspInput.from_directory(".")
+        incar = Incar.from_file("INCAR")
         self.errors = set()
         with open(self.output_filename, "r") as f:
             for line in f:
@@ -92,10 +97,10 @@ class VaspErrorHandler(ErrorHandler):
                 for err, msgs in VaspErrorHandler.error_msgs.items():
                     for msg in msgs:
                         if l.find(msg) != -1:
-			    #this checks if we want to run a charged computation
+                            #this checks if we want to run a charged computation
                             #(e.g., defects) if yes we don't want to kill it
                             #because there is a change in e- density (brmix error)
-                            if err == "brmix" and 'NELEC' in vi['INCAR']:
+                            if err == "brmix" and 'NELECT' in incar:
                                 continue
                             self.errors.add(err)
         return len(self.errors) > 0
@@ -114,14 +119,26 @@ class VaspErrorHandler(ErrorHandler):
             actions.append({"dict": "INCAR",
                             "action": {"_set": {"SYMPREC": 1e-8}}})
 
-        if "brmix" in self.errors and "NELECT" in vi["INCAR"]:
-            #brmix error always shows up after DAV steps if NELECT is specified
-            self.errors.remove("brmix")
-        if "brmix" in self.errors or "zpotrf" in self.errors:
+        if "brmix" in self.errors:
             actions.append({"dict": "INCAR",
                             "action": {"_set": {"ISYM": 0}}})
             # Based on VASP forum's recommendation, you should delete the
-            # CHGCAR and WAVECAR when dealing with these errors.
+            # CHGCAR and WAVECAR when dealing with this error.
+            actions.append({"file": "CHGCAR",
+                            "action": {"_file_delete": {'mode': "actual"}}})
+            actions.append({"file": "WAVECAR",
+                            "action": {"_file_delete": {'mode': "actual"}}})
+
+        if "zpotrf" in self.errors:
+            #Based on VASP forum's recommendation, you should
+            #decrease POTIM with this error
+            potim = float(vi["INCAR"].get("POTIM", 0.5)) / 2.0
+
+            actions.append({"dict": "INCAR",
+                            "action": {"_set": {"ISYM": 0, "POTIM": potim}}})
+            # Based on VASP forum's recommendation, you should delete the
+            # CHGCAR and WAVECAR when dealing with this error.
+
             actions.append({"file": "CHGCAR",
                             "action": {"_file_delete": {'mode': "actual"}}})
             actions.append({"file": "WAVECAR",
@@ -244,15 +261,18 @@ class MeshSymmetryErrorHandler(ErrorHandler):
               " lattices."
         try:
             v = Vasprun(self.output_vasprun)
-            if v.converged:
+            # According to VASP admins, you can disregard this error
+            # if symmetry is off
+            if v.converged or (not v.incar.get('ISYM', True)):
                 return False
+            # do not apply if job was terminated (vasprun.xml cannot be read)
+            with open(self.output_filename, "r") as f:
+                for line in f:
+                    l = line.strip()
+                    if l.find(msg) != -1:
+                        return True
         except:
             pass
-        with open(self.output_filename, "r") as f:
-            for line in f:
-                l = line.strip()
-                if l.find(msg) != -1:
-                    return True
         return False
 
     def correct(self):
@@ -297,10 +317,10 @@ class UnconvergedErrorHandler(ErrorHandler):
     def check(self):
         try:
             v = Vasprun(self.output_filename)
+            if not v.converged:
+                return True
         except:
-            return False
-        if not v.converged:
-            return True
+            pass
         return False
 
     def correct(self):
@@ -310,7 +330,7 @@ class UnconvergedErrorHandler(ErrorHandler):
                    {"dict": "INCAR",
                     "action": {"_set": {"ISTART": 1,
                                         "ALGO": "Normal",
-                                        "NELMDL": 6,
+                                        "NELMDL": -6,
                                         "BMIX": 0.001,
                                         "AMIX_MAG": 0.8,
                                         "BMIX_MAG": 0.001}}}]
@@ -329,6 +349,58 @@ class UnconvergedErrorHandler(ErrorHandler):
         return self.__name__
 
 
+class MaxForceErrorHandler(ErrorHandler):
+    """
+    Checks that the desired force convergence has been achieved. Otherwise
+    restarts the run with smaller EDIFF. (This is necessary since energy
+    and force convergence criteria cannot be set simultaneously)
+    """
+    is_monitor = False
+
+    def __init__(self, output_filename="vasprun.xml",
+                 max_force_threshold=0.5):
+        """
+        Args:
+            input_filename (str): name of the vasp INCAR file
+            output_filename (str): name to look for the vasprun
+            max_force_threshold (float): Threshold for max force for
+                restarting the run. (typically should be set to the value
+                that the creator looks for)
+        """
+        self.output_filename = output_filename
+        self.max_force_threshold = max_force_threshold
+
+    def check(self):
+        try:
+            v = Vasprun(self.output_filename)
+            max_force = max([np.linalg.norm(a) for a
+                             in v.ionic_steps[-1]["forces"]])
+            if max_force > self.max_force_threshold:
+                return True
+        except:
+            pass
+        return False
+
+    def correct(self):
+        backup(["INCAR", "KPOINTS", "POSCAR", "OUTCAR",
+                self.output_filename])
+        vi = VaspInput.from_directory(".")
+        ediff = float(vi["INCAR"].get("EDIFF", 1e-4))
+        actions = [{"file": "CONTCAR",
+                    "action": {"_file_copy": {"dest": "POSCAR"}}},
+                   {"dict": "INCAR",
+                    "action": {"_set": {"EDIFF": ediff*0.75}}}]
+        m = Modder(actions=[DictActions, FileActions])
+        for a in actions:
+            if "dict" in a:
+                vi[a["dict"]] = m.modify_object(a["action"], vi[a["dict"]])
+            elif "file" in a:
+                m.modify(a["action"], a["file"])
+        vi["INCAR"].write_file("INCAR")
+
+        return {"errors": ["MaxForce"], "actions": actions}
+
+
 class PotimErrorHandler(ErrorHandler):
     """
     Check if a run has excessively large positive energy changes.
@@ -620,3 +692,143 @@ class PBSWalltimeHandler(WalltimeHandler):
 
     def __init__(self, buffer_time=300):
         WalltimeHandler.__init__(self, None, buffer_time=buffer_time)
+
+
+class CheckpointHandler(ErrorHandler):
+    """
+    This is not an error handler per se, but rather a checkpointer. What this
+    does is that every X seconds, a STOPCAR and CHKPT will be written. This
+    forces VASP to stop at the end of the next ionic step. The files are then
+    copied into a subdir, and then the job is restarted. To use this proper,
+    max_errors in Custodian must be set to a very high value, and you
+    probably wouldn't want to use any standard VASP error handlers. The
+    checkpoint will be stored in subdirs chk_#. This should be used in
+    combiantion with the StoppedRunHandler.
+    """
+    is_monitor = True
+
+    # The CheckpointHandler should not terminate as we want VASP to terminate
+    # itself naturally with the STOPCAR.
+    is_terminating = False
+
+    def __init__(self, interval=3600):
+        """
+        Initializes the handler with an interval.
+
+        Args:
+            interval (int): Interval at which to checkpoint in seconds.
+            Defaults to 3600 (1 hr).
+        """
+        self.interval = interval
+        self.start_time = datetime.datetime.now()
+        self.chk_counter = 0
+
+    def check(self):
+        run_time = datetime.datetime.now() - self.start_time
+        total_secs = run_time.seconds + run_time.days * 3600 * 24
+        if total_secs > self.interval:
+            return True
+        return False
+
+    def correct(self):
+        content = "LSTOP = .TRUE."
+        chkpt_content = "Index: %d\nTime: \"%s\"" % (self.chk_counter,
+                                                     datetime.datetime.now())
+        self.chk_counter += 1
+
+        #Write STOPCAR
+        actions = [{"file": "STOPCAR",
+                    "action": {"_file_create": {'content': content}}},
+                   {"file": "chkpt.yaml",
+                    "action": {"_file_create": {'content': chkpt_content}}}]
+
+        m = Modder(actions=[FileActions])
+        for a in actions:
+            m.modify(a["action"], a["file"])
+
+        # Reset the clock.
+        self.start_time = datetime.datetime.now()
+
+        return {"errors": ["Checkpoint reached"], "actions": actions}
+
+    def __str__(self):
+        return "CheckpointHandler with interval %d" % self.interval
+
+
+class StoppedRunHandler(ErrorHandler):
+    """
+    This is not an error handler per se, but rather a checkpointer. What this
+    does is that every X seconds, a STOPCAR will be written. This forces VASP to
+    stop at the end of the next ionic step. The files are then copied into a
+    subdir, and then the job is restarted. To use this proper, max_errors in
+    Custodian must be set to a very high value, and you probably wouldn't
+    want to use any standard VASP error handlers. The checkpoint will be
+    stored in subdirs chk_#. This should be used in combiantion with the
+    StoppedRunHandler.
+    """
+    is_monitor = False
+
+    # The CheckpointHandler should not terminate as we want VASP to terminate
+    # itself naturally with the STOPCAR.
+    is_terminating = False
+
+    def __init__(self):
+        pass
+
+    def check(self):
+        return os.path.exists("chkpt.yaml")
+
+    def correct(self):
+        d = loadfn("chkpt.yaml")
+        i = d["Index"]
+        name = shutil.make_archive(
+            os.path.join(os.getcwd(), "vasp.chk.%d" % i), "gztar")
+
+        actions = [{"file": "CONTCAR",
+                    "action": {"_file_copy": {"dest": "POSCAR"}}}]
+
+        m = Modder(actions=[FileActions])
+        for a in actions:
+            m.modify(a["action"], a["file"])
+
+        actions.append({"Checkpoint": name})
+
+        return {"errors": ["Stopped run."],
+                "actions": actions}
+
+    def __str__(self):
+        return "StoppedRunHandler"
+
+
+class BadVasprunXMLHandler(ErrorHandler):
+    """
+    Handler to properly terminate a run when a bad vasprun.xml is found.
+    """
+
+    is_monitor = False
+
+    is_terminating = True
+
+    def __init__(self):
+        self.vasprunxml = None
+        pass
+
+    def check(self):
+        try:
+            self.vasprunxml = _get_vasprun()
+            v = Vasprun(self.vasprunxml)
+        except:
+            return True
+        return False
+
+    def correct(self):
+        return {"errors": ["Bad vasprun.xml in %s." % self.vasprunxml],
+                "actions": None}
+
+    def __str__(self):
+        return "BadVasprunXMLHandler"
+
+
+def _get_vasprun(path="."):
+    vaspruns = glob.glob(os.path.join(path, "vasprun.xml*"))
+    return sorted(vaspruns, reverse=True)[0] if vaspruns else None
\ No newline at end of file
